package org.greenstone.gdbm;

import org.greenstone.gsdl3.util.OID;
import au.com.pharos.gdbm.GdbmFile;
import au.com.pharos.packing.*;
import au.com.pharos.gdbm.GdbmException;

/** java wrapper class for gdbm - uses Java-GDBM written by Martin Pool
 * replaces gdbmclass in the old version
 */

public class GDBMWrapper {

    // Values must match those from gdbm.h - uses the definitions from GdbmFile !
    /** Indicates that the caller will just read the database.  Many
     * readers may share a database. */
    public final static int READER = GdbmFile.READER;
 
    /** The caller wants read/write access to an existing database and
     * requires exclusive access. */
    public final static int WRITER = GdbmFile.WRITER;
 
    /** The caller wants exclusive read/write access, and the database
     * should be created if it does not already exist. */
    public final static int WRCREAT = GdbmFile.WRCREAT;
 
    /** The caller wants exclusive read/write access, and the database
     * should be replaced if it already exists. */
    public final static int NEWDB = GdbmFile.NEWDB;

    protected GdbmFile db_=null;

    /** open the database filename, with mode mode - uses the constants
	above, eg GdbmFile.WRITER */
    public boolean openDatabase(String filename, int mode){
	try {
	    if (db_!=null) {
		db_.close();
	    }
	    db_ = new GdbmFile(filename, mode);
	} catch ( GdbmException e) { // the database wasn't opened or created
	    System.err.println("GDBMWrapper: couldn't open database "+filename);
	    return false;
	} 
	db_.setKeyPacking(new StringPacking());
	db_.setValuePacking(new StringPacking());
	return true;
    }

    /** cloase the database associated with this wrapper */
    public void closeDatabase() {
	try {
	    db_.close();
	} catch (GdbmException e) {
	    // should never get here - close never actually throws an exception
	    System.err.println("GDBMWrapper: error on close()");
	}
    }

    /** returns a DBInfo object containing all the name-value pairs for 
     * main_key 
     * @see DBInfo
     */
    public DBInfo getInfo(String main_key) {
	if (db_==null) {
	    return null;
	}
	String s_info;
	try {
	    s_info = (String)db_.fetch(main_key);
	} catch (GdbmException e) {
	    System.err.println("GDBMWrapper: couldn't get record");
	    return null;
	}
	if (s_info==null) {
	    // record not present
	    System.err.println("GDBMWrapper: key "+main_key+" not present in db");
	    return null;
	}
	DBInfo info = new DBInfo();

	String [] lines = s_info.split("\n");
	String key;
	String value;
	for (int i=0; i<lines.length; i++) {
	    //System.out.println("line:"+lines[i]);
	    int a = lines[i].indexOf('<');
	    int b= lines[i].indexOf('>');
	    if (a==-1 || b==-1) {
		System.err.println("error: bad format in db");
	    }
	    else {
		key=lines[i].substring(a+1, b);
		value=lines[i].substring(b+1);
		//System.out.println("key="+key+", val="+value);
		info.addInfo(key, value);

	    }
	}
	return info;
    }

    /** sets all the name-value pairs in info as the content for key key 
     * TODO - not implemented yet */
    public boolean setInfo(String key, DBInfo info) {
	if (db_==null) {
	    return false;
	}
	return true;
    }
    /** sets the key value as info 
     * TODO - not implemented yet */
    public boolean setInfo (String key, String info) {
	if (db_==null) {
	    return false;
	}
	return true;
    }
    /** deletes the entry for key 
     * TODO - not implemented yet */
    public boolean deleteKey(String key) {
	if (db_==null) {
	    return false;
	}
	return true;
    }

    // greenstone convenience methods - should these go into a separate class?

    /** converts a greenstone OID to internal docnum */
    public long oid2Docnum(String OID) {
	DBInfo info = getInfo(OID);
	long real_num = Long.parseLong(info.getInfo("docnum"));
	return real_num;
    }
    /** converts a docnum to greenstone OID */
    public String docnum2Oid(long docnum) {
	DBInfo info = getInfo(Long.toString(docnum));
	String oid = info.getInfo("section");
	return oid;
    }

    /** translates oids containing .pr, .fc, .lc, .ns, .ps into proper oids
     * a suffix is expected to be present so test before using */
    public String translateOID(String oid) {

	int p = oid.lastIndexOf('.');
	if (p != oid.length()-3) {
	    System.out.println("translateoid error: '.' is not the third to last char!!");
	    return oid;
	}

	String top = oid.substring(0, p);
	String suff = oid.substring(p+1);
	if (suff.equals("pr")) {
	    return OID.getParent(top);
	} else if (suff.equals("fc")) {
	} else if (suff.equals("lc")) {
	} else if (suff.equals("ns")) {
	} else if (suff.equals("ps")) {
	}
	return oid;
    }
}
